%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Mise en oeuvre de Dynamic Average Consensus Estimators (DACE) Proportionnels (P). 
% On considere differents graphes de communication entre les agents (n = 10)   
% et differentes entrees, selectionnables dans le modele Simulink "Simul_DACE_P.mdl"
% 
% TD3, Ex. III - Surveillance Distribuee de Systemes Multi-agents 

clear
close all
clc

%%% Parametres de simulation %%%

Tf = 20;   %% Duree de la simulation

Tc = 0.01; %% Temps d'echantillonnage

%%% Parametres des estimateurs proportionels (les parametres sont les memes pour tous les estimateurs) %%%

gamma = 2; %% "forgetting factor"  

KP = 2; %% Gain proportionnel  
     
%%% >>>> Graphe de communication (n = 10 sommets): selectionner la matrice d'adjacence A <<<< %%%        

%%% 1) Graphe chaine [lambda_2 = 0.0979] %%%
 
% A = [0 1 0 0 0 0 0 0 0 0;
%      1 0 1 0 0 0 0 0 0 0;
%      0 1 0 1 0 0 0 0 0 0;
%      0 0 1 0 1 0 0 0 0 0;
%      0 0 0 1 0 1 0 0 0 0;
%      0 0 0 0 1 0 1 0 0 0;
%      0 0 0 0 0 1 0 1 0 0;
%      0 0 0 0 0 0 1 0 1 0;
%      0 0 0 0 0 0 0 1 0 1;
%      0 0 0 0 0 0 0 0 1 0];

%%% 2) Graphe cycle [lambda_2(L) = 2*(1 - cos(pi/5)) = 0.3820] %%% 

% A = [0 1 0 0 0 0 0 0 0 1;
%      1 0 1 0 0 0 0 0 0 0;
%      0 1 0 1 0 0 0 0 0 0;
%      0 0 1 0 1 0 0 0 0 0; 
%      0 0 0 1 0 1 0 0 0 0;
%      0 0 0 0 1 0 1 0 0 0;
%      0 0 0 0 0 1 0 1 0 0;
%      0 0 0 0 0 0 1 0 1 0;
%      0 0 0 0 0 0 0 1 0 1;
%      1 0 0 0 0 0 0 0 1 0]; 
 
%%% 3) Graphe etoile (lambda_2(L) = 1) %%% 
     
A = [0 1 1 1 1 1 1 1 1 1; 
     1 0 0 0 0 0 0 0 0 0;
     1 0 0 0 0 0 0 0 0 0;
     1 0 0 0 0 0 0 0 0 0;
     1 0 0 0 0 0 0 0 0 0;
     1 0 0 0 0 0 0 0 0 0;
     1 0 0 0 0 0 0 0 0 0;
     1 0 0 0 0 0 0 0 0 0;
     1 0 0 0 0 0 0 0 0 0;
     1 0 0 0 0 0 0 0 0 0];
  
%%% 4) Graphe roue (lambda_2(L) = 1.4679) %%%

% A = [0 1 1 1 1 1 1 1 1 1;
%      1 0 1 0 0 0 0 0 0 1;
%      1 1 0 1 0 0 0 0 0 0;
%      1 0 1 0 1 0 0 0 0 0;
%      1 0 0 1 0 1 0 0 0 0;
%      1 0 0 0 1 0 1 0 0 0;
%      1 0 0 0 0 1 0 1 0 0;
%      1 0 0 0 0 0 1 0 1 0;
%      1 0 0 0 0 0 0 1 0 1;
%      1 1 0 0 0 0 0 0 1 0];
 
%%% 5) Graphe de Petersen (lambda_2(L) = 2) %%%

% A = [0 1 0 0 1 1 0 0 0 0;
%      1 0 1 0 0 0 1 0 0 0;
%      0 1 0 1 0 0 0 1 0 0;
%      0 0 1 0 1 0 0 0 1 0;
%      1 0 0 1 0 0 0 0 0 1;
%      1 0 0 0 0 0 0 1 1 0;
%      0 1 0 0 0 0 0 0 1 1;
%      0 0 1 0 0 1 0 0 0 1;
%      0 0 0 1 0 1 1 0 0 0;
%      0 0 0 0 1 0 1 1 0 0];

%%% 6) Graphe complet (lambda_2(L) = 10) %%%

% A = ones(10) - eye(10);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
D = diag(A*ones(10,1)); %%% Matrice des degres
 
L = D - A; %%% Matrice laplacienne 

Dyn_mat = -(gamma*eye(10)+KP*L); %%% Matrice dynamique de l'ensemble des estimateurs  

B = -KP*L; %% Matrice d'entree de l'ensemble des estimateurs 

%%% Initialization aleatoire des etats x0 et z0 de l'ensemble des estimateurs PI

x0 = rand(10,1);
 
%%% Lancement du modele Simulink %%%

sim('Simul_DACE_P')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Affichage des resultats %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1)
hold on
%%% Affichage de l'evolution temporelle des sorties y_1,...,y_10 des estimateurs proportionnels 
plot(T,y(:,1),'b')
plot(T,y(:,2),'g')
plot(T,y(:,3),'r')
plot(T,y(:,4),'c')
plot(T,y(:,5),'m')
plot(T,y(:,6),'y')
plot(T,y(:,7),'k--')
plot(T,y(:,8),'b--')
plot(T,y(:,9),'g--')
plot(T,y(:,10),'r--')
plot(T,mean(u,2),'linewidth',1.5,'Color','k') %% Moyenne des entrees des estimateurs proportionnels
xlabel('temps [sec.]')
ylabel('y_i(t),  i = 1,...,10','FontSize',12)
title('Ligne noire: moyenne des entrees des estimateurs proportionnels')

figure(2)
hold on
%%% Affichage de l'evolution temporelle des entrees u_1,...,u_10 des estimateurs proportionnels %%%
plot(T,u(:,1),'b')
plot(T,u(:,2),'g')
plot(T,u(:,3),'r')
plot(T,u(:,4),'c')
plot(T,u(:,5),'m')
plot(T,u(:,6),'y')
plot(T,u(:,7),'k--')
plot(T,u(:,8),'b--')
plot(T,u(:,9),'g--')
plot(T,u(:,10),'r--')
xlabel('temps [sec.]')
ylabel('u_i(t),  i = 1,...,10','FontSize',12) 
